FusionHUD = LibStub("AceAddon-3.0"):NewAddon("FusionHUD", "AceConsole-3.0");
local AceGUI = LibStub("AceGUI-3.0");

local FusionHUDBarPrototype = {};
FusionHUDBarPrototype.__index = FusionHUDBarPrototype;

local FusionHUDMasterFrame = CreateFrame("Frame", nil, UIParent);
local FusionHUDLockFrame = CreateFrame("Frame", nil, UIParent);

local _G = _G;
local FusionHUDBars = {};

local FusionHUDSettings = { 
	mainOptions = {
		locked = true;
	},
};

FusionHUDOptions = {
    name = "Fusion HUD",
    handler = FusionHUD,
    type = 'group',
    args = {
		mainOptions = {
			name = "Main Menu",
			order = 1,
			type = "group",
			desc = "Main menu for FusionHUD",
			args = {
				newBar = {
					name = "New Bar",
					type = "execute",
					desc = "Creates a new HUD bar",
					func = 	function()	FusionHUDBars[#FusionHUDBars + 1] = FusionHUDBarPrototype:New(#FusionHUDBars + 1) end,
				},
				lockAll = {
					name = "Unlock All",
					type = "execute",
					desc = "Unlocks all HUD bars for repositioning",
					func = 	function() return end,
				},
			}		
		},
		bars = {
			name = "Bar List",
			order = 2,
			type = "group",
			desc = "Listing of each HUD bar",
			args = {}
		},
    },
};

function FusionHUDBarPrototype:New(index)		--New Bar object, initializes the frames and textures
	local self = {};
    setmetatable(self, FusionHUDBarPrototype); -- Set the metatable so we used Character's __index
	self.barIndex = index;
	self.parentFrame = "FusionHUDMasterFrame";
	self.draggable = false;
	self.myPoint = "CENTER";
	self.parentPoint = "CENTER";
	self.orientation = "LEFT";
	self.textureName = "Interface\\AddOns\\FusionHUD\\Graphics\\White Straight Bar Base";
	self.containerTextureName = "Interface\\AddOns\\FusionHUD\\Graphics\\Straight Bar Glass Container";
	self.height = 400;
	self.width = 15;
	self.barType = "Health";
	self.barTarget = "player";
	self.totals = true;
	self.text = "EEEEEEE";
	self.warningColor = false;
	--Set up the bar's frame
	self.frame = CreateFrame("Frame", nil, FusionHUDMasterFrame);
	self.frame:ClearAllPoints();
	self.frame:SetPoint(self.myPoint, FusionHUDMasterFrame, self.parentPoint);
	self.frame:SetHeight(self.height);
	self.frame:SetWidth(self.width);
	self.frame:SetClampedToScreen(true);
	--Set Up the title region for the bar to enable dragging
	self.frame.titleRegion = self.frame:CreateTitleRegion();
	self.frame.titleRegion:ClearAllPoints();
	--Set up the bar texture
	self.frame.barTexture = self.frame:CreateTexture(nil, "BACKGROUND");
	self.frame.barTexture:SetTexture(self.textureName, false);
	self.frame.barTexture:ClearAllPoints();
	self.frame.barTexture:SetPoint("BOTTOM", self.frame, "BOTTOM");
	self.frame.barTexture:SetHeight(self.height);
	self.frame.barTexture:SetWidth(self.width);
	--Set up the bar background/scale texture
	self.frame.containerTexture = self.frame:CreateTexture(nil, "OVERLAY");
	self.frame.containerTexture:SetTexture(self.containerTextureName, false);
	self.frame.containerTexture:ClearAllPoints();
	self.frame.containerTexture:SetPoint("BOTTOM", self.frame, "BOTTOM");
	self.frame.containerTexture:SetHeight(self.height);
	self.frame.containerTexture:SetWidth(self.width);
	self.frame.containerTexture:SetTexCoord(0,1,1,0);
	self.frame.containerTexture:SetBlendMode("BLEND");
	--Set up bar text
	self.frame.text = self.frame:CreateFontString(nil, "LOW");
	self.frame.text:ClearAllPoints();
	self.frame.text:SetPoint("TOP", self.frame, "BOTTOM");
	self.frame.text:SetFont("Fonts\\FRIZQT__.TTF", 14, "OUTLINE");
	self.frame.text:SetJustifyH("CENTER");
	self.frame.text:SetJustifyV("TOP");
	self.frame.text:SetHeight(30);
	self.frame.text:SetWidth(150);
	
	--Set Up the draggable frame
	self.frame.dragTexture = self.frame:CreateTexture(nil, "OVERLAY");
	self.frame.dragTexture:SetTexture("Interface\\AddOns\\FusionHUD\\Graphics\\whitebackdropbase", true);
	self.frame.dragTexture:ClearAllPoints();
	
	--Set Up new bar's options table
	FusionHUDOptions.args.bars.args["bar" .. self.barIndex] = {};
	FusionHUDOptions.args.bars.args["bar" .. self.barIndex].name = "Bar " .. self.barIndex;
	FusionHUDOptions.args.bars.args["bar" .. self.barIndex].type = "group";
	FusionHUDOptions.args.bars.args["bar" .. self.barIndex].order = self.barIndex;
	FusionHUDOptions.args.bars.args["bar" .. self.barIndex].args = {
		header = {
			name = "Bar " .. self.barIndex,
			order = 1,
			type = "header"
		},		
		lock = {
			name = "Locked",
			order = 2,
			type = "toggle",
			desc = "Unlocks the bar for repositioning",
			get = 	function() return not FusionHUDBars[index].draggable end;
			set = 	function() 
						FusionHUDBars[self.barIndex].draggable = true;
						FusionHUDLockFrame:Show();
						InterfaceOptionsFrame:Hide();
						GameMenuFrame:Hide();
					end;
		},
		barType = {
			name = "Bar Type",
			order = 4,
			type = "select",
			desc = "The bar can track either Health or Power(Mana/Runic Power/Energy/Focus)",
			style = "dropdown",
			values = {
				Power = "Power", 
				Health = "Health"
			},
			get = 	function() return FusionHUDBars[self.barIndex].barType end,
			set = 	function(key, value) FusionHUDBars[self.barIndex].barType = value end
		},
		barTarget = {
			name = "Bar Target",
			order = 3,
			type = "select",
			desc = "The unit that the bar will track",
			style = "dropdown",
			values = {
				player = "Player",
				pet = "Pet",
				playertarget = "Player target",
				playertargettarget = "Target of Target",
				pettarget = "Pet Target"
			},
			get = 	function() return FusionHUDBars[self.barIndex].barTarget end,
			set = 	function(key, value) FusionHUDBars[self.barIndex].barTarget = value end
		},
		spacer1 = {
			name = "___________________________",
			type = "header",
			order = 99
		},
		deleteBar = {
			name = "Delete Bar",
			order = 100,
			type = "execute",
			desc = "Delete the current bar",
			func = 	function()

					end	
		},				
	}
	return self;
end

function FusionHUD:OnInitialize()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("FusionHUD", FusionHUDOptions);
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("FusionHUD", "FusionHUD");
	
	FusionHUDMasterFrame:SetClampedToScreen(true);
	FusionHUDMasterFrame:SetPoint("CENTER",UIParent,"CENTER");
	FusionHUDMasterFrame:SetFrameStrata("LOW");
	FusionHUDMasterFrame:SetHeight(400);
	FusionHUDMasterFrame:SetWidth(400);
	FusionHUDMasterFrame:Show();
	FusionHUDMasterFrame:SetAlpha(1);

	FusionHUDLockFrame:SetClampedToScreen(true);
	FusionHUDLockFrame:ClearAllPoints();
	FusionHUDLockFrame:SetPoint("TOP", UIParent, "TOP", 0, -50);
	FusionHUDLockFrame:SetFrameStrata("FULLSCREEN");
	FusionHUDLockFrame:SetHeight(100);
	FusionHUDLockFrame:SetWidth(200);
	FusionHUDLockFrame.texture = FusionHUDLockFrame:CreateTexture(nil, "BACKGROUND");
	FusionHUDLockFrame.texture:SetAllPoints(FusionHUDLockFrame);
	FusionHUDLockFrame.texture:SetTexture("Interface\\AddOns\\FusionHUD\\Graphics\\whitebackdropbase", true);
	FusionHUDLockFrame.texture:SetVertexColor(0.1, 0.1, 0.1, 0.75);
	FusionHUDLockFrame.texture:SetTexCoord(0,1,0,1)
	FusionHUDLockFrame.text = FusionHUDLockFrame:CreateFontString(nil, "LOW")
	FusionHUDLockFrame.text:SetPoint("TOP", FusionHUDLockFrame, "TOP",0, -20);
	FusionHUDLockFrame.text:SetFont("Fonts\\FRIZQT__.TTF", 12, "BOLD");
	FusionHUDLockFrame.text:SetJustifyH("CENTER");
	FusionHUDLockFrame.text:SetJustifyV("TOP");
	FusionHUDLockFrame.text:SetHeight(30);
	FusionHUDLockFrame.text:SetWidth(150);
	FusionHUDLockFrame.text:SetText("FusionHUD");	
	
	FusionHUDLockFrame.lockButton = CreateFrame("Button", "lockButton", FusionHUDLockFrame, "UIPanelButtonTemplate")
	FusionHUDLockFrame.lockButton:SetPoint("BOTTOM", FusionHUDLockFrame, "BOTTOM", 0, 25);
	FusionHUDLockFrame.lockButton:SetWidth(80);
	FusionHUDLockFrame.lockButton:SetHeight(25);
	FusionHUDLockFrame.lockButton:SetText("Lock");
	FusionHUDLockFrame.lockButton:SetFrameStrata("FULLSCREEN_DIALOG");
	FusionHUDLockFrame.lockButton:SetScript('OnClick', FusionHUD.Relocker);
	FusionHUDLockFrame:Hide();
	
	FusionHUDMasterFrame:SetScript("OnUpdate", 	function() FusionHUD:OnUpdate()	end); 	
end
	
function FusionHUD:Relocker()
	local i = 1;
	while FusionHUDBars[i]  do
		FusionHUDBars[i].draggable = false;
		i = i + 1;
	end
	InterfaceOptionsFrame:Show();
end

function FusionHUDBarPrototype:Update()	-- The bar update function
	if self.draggable == true then
		FusionHUD:Print("Bar " .. self.barIndex .. " is draggable");
		self.frame.titleRegion:SetAllPoints(self.frame);
		self.frame:SetBackdrop({bgFile = "Interface\\AddOns\\FusionHUD\\Graphics\\whitebackdropbase", tile = true, tilesize = self.width});
		self.frame:SetBackdropColor(0.6, 0.6, 0.6, 0.75);
		self.frame:Show();
		self.frame:SetAlpha(1);
		self.frame:EnableMouse(true);
		self.frame:SetMovable(true);
		return
	else
		self.frame.titleRegion:ClearAllPoints();
		self.frame:SetBackdrop({nil});
		self.frame:EnableMouse(false);
		self.frame:SetMovable(false);
	end
	if UnitName(self.barTarget) == nil then
		self.frame:Hide();
		return
	else
		self.frame:Show();
		if self.barType == "Health" then  --check for type of bar, power or health
			--set bar height and text for a health bar
			local health = UnitHealth(self.barTarget) .. "/" .. UnitHealthMax(self.barTarget);
			self.frame.barTexture:SetHeight(400*UnitHealth(self.barTarget)/UnitHealthMax(self.barTarget));
			self.frame.barTexture:SetTexCoord(0,1,UnitHealth(self.barTarget)/UnitHealthMax(self.barTarget),0);
			self.frame.barTexture:SetVertexColor(0,1,0,1);
			self.frame.text:SetVertexColor(0,1,0,1);
			self.frame.text:SetText(health);
		elseif self.barType == "Power" then
			local powerType = UnitPowerType(self.barTarget);
			local power = UnitMana(self.barTarget) .. "/" .. UnitManaMax(self.barTarget)
			--first color the bar the appropriate color
			if powerType == 1 then
				self.frame.barTexture:SetVertexColor(0.8,0,0,1);
				self.frame.text:SetVertexColor(0.8,0,0,1);
			elseif powerType == 6 then
				self.frame.barTexture:SetVertexColor(0.6,0,0,1);
				self.frame.text:SetVertexColor(0.6,0,0,1);
			elseif powerType == 0 then
				self.frame.barTexture:SetVertexColor(0,0,0.8,1);
				self.frame.text:SetVertexColor(0,0,0.8,1);
			elseif powerType == 2 then
				self.frame.barTexture:SetVertexColor(0.8,0.3,0,1);
				self.frame.text:SetVertexColor(0.8,0.3,0,1);
			else
				self.frame.barTexture:SetVertexColor(1,1,0,1);
				self.frame.text:SetVertexColor(1,1,0,1);
			end
			--set bar height and text
			if (400*UnitMana(self.barTarget)/UnitManaMax(self.barTarget)) == 0 then
				self.frame.barTexture:SetHeight(1);
			else
				self.frame.barTexture:SetHeight(400*UnitMana(self.barTarget)/UnitManaMax(self.barTarget));
			end
			self.frame.barTexture:SetTexCoord(0,1,UnitMana(self.barTarget)/UnitManaMax(self.barTarget),0);
			self.frame.text:SetText(power);
		end
	end	
end

function FusionHUD:OnUpdate()
	local i = 1;
	while FusionHUDBars[i]  do
		FusionHUDBars[i]:Update();
		i = i + 1;
	end
end